// ***********************************************
// This is the source of an sample implementation
// of the E-LAB Mega8 Terminal
// The hardware is connected to the EvaBoardI
// through a TWI Master/Slave connection
// where this implementation is the Slave
// ***********************************************

program mega8term;

{ $W+ Warnings}            {Warnings off}

Device = mega8, VCC = 5;

Define_Fuses
  LockBits0 = [];
  FuseBits0 = [CKSEL0, CKSEL1, CKSEL3, SUT0, BODEN];
  FuseBits1 = [];
  ProgMode  = SPI;

Import SysTick, LCDport, BeepPort, MatrixPort, TWIslave;

From System Import Processes, Pipes;

Define
  ProcClock      = 8000000;        {Hertz}
  SysTick        = 1;              {msec}
  StackSize      = $0064, iData;
  FrameSize      = $0064, iData;
  Scheduler      = iData;
  TWImode        = Handshake;      {HandShake or Transparen}
  TWIbuffer      = 23, iData;      {buffer/packet size}
  TWIaddr        = 02;             {default slave address}
  LCDport        = PortD;
  LCDtype        = 66712;
  LCDrows        = 4;              {rows}
  LCDcolumns     = 20;             {columns per line}
  MatrixRow      = PortB, 0;       {use PortB, start with bit0}
  MatrixCol      = PinC, 0;        {use PinC, start with bit4}
  MatrixType     = 3, 4;           {4 Rows at PortB, 4 Columns at PinC}
  BeepPort       = PortB, 6;
  LCDBarGraph1   = LCDPort;

Implementation

{$IDATA}

{--------------------------------------------------------------}
{ Type Declarations }
type
  tTermCmd       = (BusyState, KeybState, DispCmd, SoundCmd, KeybFlush, BarInit);

  tLCcmd         = (lcClr, lcHome, lcXY, lcStr, lcClrLn, lcClrEOL, lcBar, lcCurs);
  tLCcmdSet      = BitSet of tLCcmd;
  t_Display      = Record
                     Cmd            : tLCcmdSet;
                     x              : Byte;
                     y              : Byte;
                     Str            : String[20];
                   end;

  tBeepCmd       = (bpOff, bpErr, bpClick, bpLH, bpHL, bpSir0, bpSir1, bpOut);
  t_Sound        = Record
                     cmd            : tBeepCmd;
                     Frequ          : Word;
                     Ticks          : Byte;
                   end;
                   
  tBusy          = (lcBusy, bpBusy);
  tBusyState     = set of tBusy;
  
  tKeys          = BitSet of Keys;   (* enumeration "Keys" imported with "MatrixPort" *)
  
  tTermState     = record
                     BusyState : tBusyState;
                     KeyStates : tKeys;
                   end;

{--------------------------------------------------------------}
{ Const Declarations }


{--------------------------------------------------------------}
{ Var Declarations }
{$IDATA}

Var
  Display_Pipe   : Pipe[4] of t_Display;
  Sound_Pipe     : Pipe[15] of t_Sound;
  Keyboard_Pipe  : Pipe[15] of tKeys;
  
  reqKeys        : boolean;
  reqBusy        : tBusyState;

  // TWI Rx Buffer
  {$NOOVRCheck}                                       // A lot of Var's //
  TWI_Display[@TWIRxBuffer[0]]: t_Display;
  {$NOOVRCheck}
  TWI_Sound[@TWIRxBuffer[0]] : t_Sound;
  
  // TWI Tx Buffer
  {$NOOVRCheck}
  TWI_TermState[@TWITxBuffer[0]]  : tTermState;


{--------------------------------------------------------------}
{ functions }

process Handle_sound(20, 32 : iData, 5, resumed);         // Plays the Sound //
Var
  Buffer         : t_Sound;
begin
  WaitPipe(Sound_Pipe);
  PipeRecv(Sound_Pipe, Buffer);
  Lock(self);
  Case Buffer.cmd of
    bpErr   : Beepouterr;
            |
    bpClick : beepClick;
            |
    bpLH    : BeepoutLh;
            |
    bpHL    : BeepoutHl;
            |
    bpSir0  : BeepSiren(0, Buffer.Ticks);
            |
    bpSir1  : BeepSiren(1, Buffer.Ticks);
            |
    bpOut   : BeepOut(Buffer.Frequ, Buffer.Ticks);
            |
  endcase;
  UnLock(self);
end;

process Handle_Display(20, 32 : iData, 5, resumed);            // Show the Display //
Var
  Buffer         : t_Display;
begin
  WaitPipe(Display_Pipe);
  PipeRecv(Display_Pipe, Buffer);

  if lcCurs in Buffer.Cmd then           // Cursor invisible
    LCDcursor(boolean(buffer.x), boolean(Buffer.y));
  End_If;
  If lcClr in Buffer.Cmd then               // Clear the Display //
    LCDclr;
  End_If;
  If lcHome in Buffer.cmd then              // Home //
    LCDHome;
  End_If;
  If lcXY in buffer.cmd then                // Set the Cursor To XY //
    LCDXY(buffer.x, Buffer.y);
  End_If;
  if lcClrLn in Buffer.Cmd then             // Clear the entire line
    LCDclrLine(Buffer.y);
  End_If;
  If lcStr in Buffer.cmd then               //Write the String)
    Write(LCDOut, Buffer.str);
  End_If;
  if lcClrEOL in Buffer.Cmd then            // Clear to end of line
    LCDclrEol;
  End_If;
  If lcBar in Buffer.cmd then               //Write the String)
    LCDBarOut1(Buffer.x);
  End_If;
end;

process Handle_Keyboard(20, 32 : iData, 1, resumed);  // Keyboard //
Var
  KeySet         : tKeys;            (* declare a word/BitSet variable compatible to "ReadKeyBoard *)
begin
  WaitSema(KeyBoard);
  KeySet:= ReadKeyBoard;
  Repeat
  Until PipeSend(Keyboard_Pipe, KeySet);
end;

{--------------------------------------------------------------}
{ Main Program }
{$IDATA}

begin
  Twisetslaveaddr($02);
  LCDON;
  Start_Processes;
  clearkeyboard;
  loop
    if TWIgetRxStat then
      case TWIgetCmd of
        BusyState : reqKeys:= false;
                  |
        KeybState : reqKeys:= true;
                  |
        DispCmd   : Repeat
                    Until PipeSend(Display_Pipe, TWI_Display);
                  |
        SoundCmd  : Repeat
                    Until PipeSend(Sound_Pipe, TWI_Sound);
                  |
        KeybFlush : PipeFlush(Keyboard_Pipe);
                  |
        BarInit   : LCDBarInit_P;
                    LCDBarSet1 (TWIRXBuffer[0], TWIRXBuffer[1], TWIRXBuffer[2], TWIRXBuffer[3]);
                  |
      End_Case;
      TWIsetRdy(True);
    elsif TWIgetTxStat then
      if reqKeys then
        If PipeStat(Keyboard_Pipe) <> 0 Then
          TWI_TermState.KeyStates:= PipeRecv(Keyboard_Pipe);
        else
          TWI_TermState.KeyStates:= ReadKeyBoard;
        end_If;
      endif;
      TWIsetRdy(True);
    end_if;
    reqBusy:= [];
    if PipeStat(Display_Pipe) > 0 then
      reqBusy:= [lcBusy];
    endif;
    if PipeStat(Sound_Pipe) > 0 then
      Incl(reqBusy, [bpBusy]);
    endif;
    TWI_TermState.BusyState:= reqBusy;
  endloop;
end mega8term.


